/*
 * Copyright (C) 2006 Alex Aranda
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package com.fleax.ant.comments;

import java.io.*;
import java.util.Collection;
import java.util.Iterator;
import org.apache.commons.io.FileUtils;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/**
 * Ant task to add a header to a set of java files
 */
public class Commenter extends Task
{

    public Commenter()
    {
    	// Nothing to do
    }

    @Override
	public void execute()
        throws BuildException
    {
        super.execute();
        File pathFile = new File(path);
        Collection<?> filesToComment = FileUtils.listFiles(pathFile, new String[] {
            "java"
        }, true);
        String comment = null;
        try
        {
            comment = getComment();
        }
        catch(IOException e)
        {
            throw new BuildException("Could not read comment");
        }
        for(Iterator<?> it = filesToComment.iterator(); it.hasNext();)
        {
            File f = (File)it.next();
            try
            {
                editFile(f, comment);
            }
            catch(Exception e)
            {
                throw new BuildException(e);
            }
        }

        System.out.println((new StringBuilder("Commented ")).append(filesToComment.size()).append(" files").toString());
    }

    private void editFile(File f, String comment)
        throws IOException
    {
        File newFile = new File((new StringBuilder(String.valueOf(f.getAbsolutePath()))).append(".temp").toString());
        FileWriter fw = new FileWriter(newFile);
        FileReader fr = new FileReader(f);
        BufferedReader br = new BufferedReader(fr);
        fw.append(comment);
        String line = null;
        boolean startToCopy = false;
        while((line = br.readLine()) != null) 
        {
            if(!startToCopy && line.toLowerCase().contains("package"))
            {
                startToCopy = true;
                fw.append("\n");
            }
            if(startToCopy)
                fw.append((new StringBuilder(String.valueOf(line))).append("\n").toString());
        }
        fw.flush();
        fw.close();
        br.close();
        f.delete();
        newFile.renameTo(f);
    }

    private String getComment()
        throws IOException
    {
        FileReader fr = new FileReader(commentFile);
        BufferedReader br = new BufferedReader(fr);
        StringBuffer strBuffer = new StringBuffer();
        String tmp;
        while((tmp = br.readLine()) != null) 
            strBuffer.append((new StringBuilder(String.valueOf(tmp))).append("\n").toString());
        return strBuffer.toString();
    }

    public void setPath(String path)
    {
        this.path = path;
    }

    public void setCommentFile(String commentFile)
    {
        this.commentFile = commentFile;
    }

    private String path;
    private String commentFile;
}